@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 import json
+from random import shuffle
 
 import matplotlib
 import matplotlib.pyplot as plt
@@ -24,18 +25,18 @@ except ImportError:
 raw_themes_data = pkg_resources.open_text(data, "themes.json")
 themes = json.load(raw_themes_data)
 
-matplotlib.use("Agg")
-
 
 class Attractor(DES):
-
     bgcolor = None
     palette = None
     fig, ax = None, None
 
+    _update_func = None
+    _points = None
+    _init_func = None
+
     def __init__(self, attractor, **kwargs):
         self.attr = ATTRACTOR_PARAMS[attractor]
-        self._data_len = None
         self.init_coord = kwargs.get("init_coord", self.attr["init_coord"])
         self.params = {
             self.attr["params"][i]: kwargs.get(
@@ -66,7 +67,11 @@ class Attractor(DES):
 
     @staticmethod
     def list_attractors():
-        return [x for x in dir(DES.__mro__[1]) if not x.startswith("_")]
+        return list(ATTRACTOR_PARAMS.keys())
+
+    @staticmethod
+    def list_params(attr):
+        return ATTRACTOR_PARAMS[attr]["params"]
 
     @classmethod
     def set_theme(cls, theme, bgcolor, palette):
@@ -76,6 +81,8 @@ class Attractor(DES):
         elif all(v is None for v in [bgcolor, palette]) and theme is not None:
             palette_temp = list(theme.values())
             palette_temp.remove(theme["background"])
+            palette_temp.insert(0, palette_temp.pop(-1))
+            shuffle(palette_temp)
             cls.bgcolor = theme["background"]
             cls.palette = palette_temp
         else:
@@ -140,14 +147,19 @@ class Attractor(DES):
         Attractor._wrap_set(objs, kwargs)
         colors = cls.cmap(np.linspace(0, 1, len(objs)))
 
+        linekwargs = kwargs.get("linekwargs", {})
+        pointkwargs = kwargs.get("pointkwargs", {})
+
         lines = sum(
             [
-                cls.ax.plot([], [], [], "-", c=c, linewidth=1, antialiased=True)
+                cls.ax.plot([], [], [], "-", c=c, antialiased=True, **linekwargs)
                 for c in colors
             ],
             [],
         )
-        pts = sum([cls.ax.plot([], [], [], "o", c=c) for c in colors], [])
+        pts = sum(
+            [cls.ax.plot([], [], [], "o", c=c, **pointkwargs) for c in colors], []
+        )
 
         def init():
             for line, pt in zip(lines, pts):
@@ -162,23 +174,34 @@ class Attractor(DES):
             for line, pt, k in zip(lines, pts, objs):
                 line.set_data_3d(k.X[:i], k.Y[:i], k.Z[:i])
                 pt.set_data_3d(k.X[i], k.Y[i], k.Z[i])
-            cls.ax.view_init(0.005 * i, 0.05 * i)
+            cls.ax.view_init(
+                kwargs.get("elevationrate", 0.005) * i,
+                kwargs.get("azimuthrate", 0.05) * i,
+            )
             return lines + pts
 
         points = len(max(objs).X)
-        return update, points, init
+        cls._update_func = update
+        cls._init_func = init
+        cls._points = points
+        return cls
 
     @classmethod
     def set_animate_gradient(cls, obj, **kwargs):
 
         Attractor._wrap_set([obj], kwargs)
 
-        line = Line3DCollection([], cmap=cls.cmap)
+        linekwargs = kwargs.get("linekwargs", {})
+        pointkwargs = kwargs.get("pointkwargs", {})
+
+        val = getattr(obj, kwargs.get("gradientaxis", "Z"))
+
+        line = Line3DCollection([], cmap=cls.cmap, **linekwargs)
         cls.ax.add_collection3d(line)
 
-        (pt,) = cls.ax.plot([], [], [], "o")
-        line.set_array(np.array(obj.Z))
-        colors = line.to_rgba(obj.Z)
+        (pt,) = cls.ax.plot([], [], [], "o", **pointkwargs)
+        line.set_array(np.array(val))
+        colors = line.to_rgba(val)
 
         def init():
             line.set_segments([])
@@ -196,28 +219,38 @@ class Attractor(DES):
             line.set_segments(segs)
             pt.set_data_3d([obj.X[i]], [obj.Y[i]], [obj.Z[i]])
             pt.set_color(colors[i])
-            cls.ax.view_init(0.005 * i, 0.05 * i)
+            cls.ax.view_init(
+                kwargs.get("elevationrate", 0.005) * i,
+                kwargs.get("azimuthrate", 0.05) * i,
+            )
             return line, pt
 
         points = len(obj.X)
-        return update, points, init
+        cls._update_func = update
+        cls._init_func = init
+        cls._points = points
+        return cls
 
     @classmethod
-    def animate(cls, update, points, init=None, **kwargs):
+    def animate(cls, **kwargs):
         if kwargs.get("live", False):
-            _ = animation.FuncAnimation(
+            anim = animation.FuncAnimation(
                 cls.fig,
-                update,
-                init_func=init,
+                cls._update_func,
+                init_func=cls._init_func,
                 interval=1000 / kwargs.get("fps", 60),
                 blit=False,
             )
-            plt.show()
+            if kwargs.get("show", True):
+                plt.show()
+            else:
+                return anim
         else:
+            matplotlib.use("Agg")
             ffmpeg_video(
                 cls.fig,
-                update,
-                points,
+                cls._update_func,
+                cls._points,
                 kwargs.get("fps", 60),
                 kwargs.get("outf", "output.mp4"),
             )
@@ -227,12 +260,17 @@ class Attractor(DES):
 
         Attractor._wrap_set([obj], kwargs)
 
-        line = Line3DCollection([], cmap=cls.cmap)
+        linekwargs = kwargs.get("linekwargs", {})
+        pointkwargs = kwargs.get("pointkwargs", {})
+
+        val = getattr(obj, kwargs.get("gradientaxis", "Z"))
+
+        line = Line3DCollection([], cmap=cls.cmap, **linekwargs)
         cls.ax.add_collection3d(line)
 
-        (pt,) = cls.ax.plot([], [], [], "o")
-        line.set_array(np.array(obj.Z))
-        colors = line.to_rgba(obj.Z)
+        (pt,) = cls.ax.plot([], [], [], "o", **pointkwargs)
+        line.set_array(np.array(val))
+        colors = line.to_rgba(val)
 
         pts = (
             np.array([obj.X[:index], obj.Y[:index], obj.Z[:index]])
@@ -243,7 +281,6 @@ class Attractor(DES):
         line.set_segments(segs)
         pt.set_data_3d([obj.X[index]], [obj.Y[index]], [obj.Z[index]])
         pt.set_color(colors[index])
-        # cls.ax.view_init(0.005 * index, 0.05 * index)
         cls.fig.canvas.draw()
         return cls.ax
 
@@ -253,18 +290,22 @@ class Attractor(DES):
         Attractor._wrap_set(objs, kwargs)
         colors = cls.cmap(np.linspace(0, 1, len(objs)))
 
+        linekwargs = kwargs.get("linekwargs", {})
+        pointkwargs = kwargs.get("pointkwargs", {})
+
         lines = sum(
             [
-                cls.ax.plot([], [], [], "-", c=c, linewidth=1, antialiased=True)
+                cls.ax.plot([], [], [], "-", c=c, antialiased=True, **linekwargs)
                 for c in colors
             ],
             [],
         )
-        pts = sum([cls.ax.plot([], [], [], "o", c=c) for c in colors], [])
+        pts = sum(
+            [cls.ax.plot([], [], [], "o", c=c, **pointkwargs) for c in colors], []
+        )
 
         for line, pt, k in zip(lines, pts, objs):
             line.set_data_3d(k.X[:index], k.Y[:index], k.Z[:index])
             pt.set_data_3d(k.X[index], k.Y[index], k.Z[index])
-        # cls.ax.view_init(0.005 * i, 0.05 * i)
         cls.fig.canvas.draw()
         return cls.ax
